package com.gic.enterprise.service.outer;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.dto.BillingPayInfoDTO;
import com.gic.enterprise.dto.InvoiceDTO;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.BillListQueryQO;
import com.gic.enterprise.service.BillingPayInfoApiService;
import com.gic.enterprise.service.BillingPayInfoService;
import com.gic.enterprise.service.EnterpriseService;
import com.gic.redis.data.util.RedisUtil;
import com.gic.thirdparty.api.dto.ScanPayQueryResData;
import com.gic.thirdparty.api.dto.ScanPayReqDataDTO;
import com.gic.thirdparty.api.service.Pay4WXService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;

@Service("billingPayInfoApiService")
public class BillingPayInfoApiServiceImpl implements BillingPayInfoApiService {
    @Autowired
    private BillingPayInfoService billingPayInfoService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private Pay4WXService pay4WXService;

    @Override
    public ServiceResponse<Page<BillingPayInfoDTO>> listBill(BillListQueryQO params) {
        com.github.pagehelper.Page page = billingPayInfoService.listBilling(params);
        Page<BillingPayInfoDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, BillingPayInfoDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<InvoiceDTO> getInvoiceInfoFromRedis(Integer enterpriseId) {
        TabEnterprise enterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (enterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID有误");
        }
        InvoiceDTO value = (InvoiceDTO) RedisUtil.getCache("enterprise:billing:invoice:" + enterpriseId);
        if (value == null) {
            return ServiceResponse.success(new InvoiceDTO());
        }
        return ServiceResponse.success(value);
    }

    @Override
    public ServiceResponse<BillingPayInfoDTO> savePrePayInfo(Integer enterpriseId, String serialNumber, Double totalFee, Integer payType, Integer buyType) {
        if (PayTypeEnum.WE_CHAT_PAY.getCode() == payType) {
            // 微信支付
            BillingPayInfoDTO billingPayInfoDTO = saveWxPrePay(enterpriseId, serialNumber, totalFee, buyType);
            if (billingPayInfoDTO != null) {
                return ServiceResponse.success(billingPayInfoDTO);
            } else {
                return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "调用微信支付失败");
            }
        } else if (PayTypeEnum.ALI_PAY.getCode() == payType) {
            // 支付宝
        } else if (PayTypeEnum.OFFLINE_PAY.getCode() == payType) {
            return ServiceResponse.success(saveOfflinePay(enterpriseId, serialNumber, totalFee, buyType));
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ErrorCode.PARAMETER_ERROR.getMsg());
    }

    @Override
    public ServiceResponse<Integer> queryPayStatus(Integer enterpriseId, String serialNumber, Integer payType) {
        if (PayTypeEnum.WE_CHAT_PAY.getCode() == payType) {
            // 微信支付
            ServiceResponse<ScanPayQueryResData> serviceResponse = pay4WXService.scanPayOrderQuery(serialNumber);
            if (serviceResponse.isSuccess()) {
                ScanPayQueryResData resData = serviceResponse.getResult();
                if (StringUtils.equalsIgnoreCase(resData.getReturn_code(), "SUCCESS")) {
                    Integer i = billingPayInfoService.updatePayInfoBySerialNumber(
                            enterpriseId,
                            serialNumber,
                            StringUtils.equalsIgnoreCase(resData.getResult_code(), "SUCCESS") ? 1 : 0,
                            resData.getErr_code(),
                            resData.getErr_code_des());
                }
                return ServiceResponse.success();
            } else {
                return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
            }

        } else if (PayTypeEnum.ALI_PAY.getCode() == payType) {
            // 支付宝
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ErrorCode.PARAMETER_ERROR.getMsg());

    }

    private BillingPayInfoDTO saveWxPrePay(Integer enterpriseId, String serialNumber, Double totalFee, Integer buyType) {
        Date startDate = new Date();
        String startDateFormat = DateUtil.formatString(startDate, DateUtil.FORMAT_DATETIME_14);
        // 生成付款码
        ScanPayReqDataDTO scanPayReqDataDTO = new ScanPayReqDataDTO(
                "充值",
                enterpriseId.toString(),
                serialNumber,
                new BigDecimal(totalFee).add(new BigDecimal(100)).intValue(),
                startDateFormat,
                "",
                "NATIVE"
        );
        ServiceResponse<String> serviceResponse = pay4WXService.createPrePaymentOrderForWX(scanPayReqDataDTO);
        if ("SUCCESS".equals(serviceResponse.getCode())) {
            // 生成支付记录
            BillingPayInfoDTO billingPayInfoDTO = new BillingPayInfoDTO();
            billingPayInfoDTO.setEnterpriseId(enterpriseId);
            billingPayInfoDTO.setSerialNumber(serialNumber);
            billingPayInfoDTO.setPayType(PayTypeEnum.WE_CHAT_PAY.getCode());
            billingPayInfoDTO.setTimeStart(startDateFormat);
            billingPayInfoDTO.setCodeUrl(serviceResponse.getResult());
            billingPayInfoDTO.setTotalFeePaid(totalFee);
            billingPayInfoDTO.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
            billingPayInfoDTO.setCreateTime(new Date());
            billingPayInfoDTO.setBuyType(buyType);
            Integer payInfoId = billingPayInfoService.savePayInfo(billingPayInfoDTO);
            billingPayInfoDTO.setPayInfoId(payInfoId);
            return billingPayInfoDTO;
        } else {
            return null;
        }
    }

    private BillingPayInfoDTO saveOfflinePay(Integer enterpriseId, String serialNumber, Double totalFee, Integer buyType) {
        // 生成支付记录
        BillingPayInfoDTO billingPayInfoDTO = new BillingPayInfoDTO();
        billingPayInfoDTO.setEnterpriseId(enterpriseId);
        billingPayInfoDTO.setSerialNumber(serialNumber);
        billingPayInfoDTO.setPayType(PayTypeEnum.WE_CHAT_PAY.getCode());
        billingPayInfoDTO.setTimeStart(DateUtil.formatString(new Date(), DateUtil.FORMAT_DATETIME_14));
        billingPayInfoDTO.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        billingPayInfoDTO.setCreateTime(new Date());
        billingPayInfoDTO.setBuyType(buyType);
        Integer payInfoId = billingPayInfoService.savePayInfo(billingPayInfoDTO);
        billingPayInfoDTO.setPayInfoId(payInfoId);
        return billingPayInfoDTO;
    }

}
